import java.io.File;

import javax.swing.JOptionPane;

public class MCDittyJarSwapper {

	private static File oldMinecraftJar;
	private static File updatedMinecraftJar;
	private static File backupMinecraftJar;

	/**
	 * @param args
	 *            The minecraft.jar's folder that needs to be updated. If empty,
	 *            will try to find the default ".minecraft/bin" folder.
	 * @throws InterruptedException
	 */
	public static void main(String[] args) throws InterruptedException {
		if (args.length > 0) {
			String arg = "";
			for (String s : args) {
				arg += s + " ";
			}
			arg = arg.trim();

			oldMinecraftJar = new File(arg + "minecraft.jar");
			updatedMinecraftJar = new File(arg + "minecraft.updatedMCDitty.jar");

			backupMinecraftJar = new File(arg + "minecraft.mcdittyUpdateBackup.jar");
		} else {

			oldMinecraftJar = new File(getMinecraftDir() + File.separator + "bin" + File.separator + "minecraft.jar");
			updatedMinecraftJar = new File(getMinecraftDir() + File.separator + "bin" + File.separator + "minecraft.updatedMCDitty.jar");

			backupMinecraftJar = new File(getMinecraftDir() + File.separator + "bin" + File.separator + "minecraft.mcdittyUpdateBackup.jar");
		}

		if (!oldMinecraftJar.exists()) {
			JOptionPane.showMessageDialog(null, "<html>You do not appear to have a minecraft.jar<br>" + "<b>Could not update MCDitty automatically.</b><br>"
					+ "If you wish, you may still easily update it yourself.<br><br>" + "Looked for minecraft.jar at<br> " + oldMinecraftJar, "MCDitty Jar Swapper", JOptionPane.WARNING_MESSAGE);
			System.exit(1);
		}

		if (!updatedMinecraftJar.exists()) {
			JOptionPane.showMessageDialog(null, "<html>You do not appear to have an updated minecraft.jar<br>" + "<b>Could not update MCDitty automatically.</b><br>"
					+ "If you wish, you may still easily update it yourself.<br><br>" + "Looked for minecraft.updatedMCDitty.jar at<br> " + updatedMinecraftJar, "MCDitty Jar Swapper", JOptionPane.WARNING_MESSAGE);
			System.exit(2);
		}

		// After verifying that swap can occur, wait for minecraft to close
		Thread.sleep(2000);

		// Remove any old backup
		if (backupMinecraftJar.exists()) {
			backupMinecraftJar.delete();
		}

		// Attempt to swap out old minecraft jar
		// Try for 20 seconds
		int counter = 10;
		while (!oldMinecraftJar.renameTo(backupMinecraftJar)) {
			Thread.sleep(2000);
			counter--;
			if (counter <= 0) {
				JOptionPane.showMessageDialog(null, "<html>Cannot rename your minecraft.jar to minecraft.mcdittyUpdateBackup.jar<br>" + "<b>Could not update MCDitty automatically</b>.<br>"
						+ "If you wish, you may still easily update it yourself.<br><br>" + "Looked for minecraft.jar at<br> " + oldMinecraftJar, "MCDitty Jar Swapper", JOptionPane.WARNING_MESSAGE);
				System.exit(3);
				break;
			}
		}

		// Attempt to swap in new minecraft.jar
		counter = 5;
		while (!updatedMinecraftJar.renameTo(oldMinecraftJar)) {
			Thread.sleep(2000);
			counter--;
			if (counter <= 0) {
				JOptionPane.showMessageDialog(null, "<html>Cannot rename minecraft.mcdittyUpdateBackup.jar to minecraft.jar<br>" + "<b>Could not update MCDitty automatically.</b><br>"
						+ "If you wish, you may still easily update it yourself.<br><br>" + "Looked for minecraft.mcdittyUpdateBackup.jar at<br> " + updatedMinecraftJar, "MCDitty Jar Swapper", JOptionPane.WARNING_MESSAGE);
				System.exit(4);
				break;
			}
		}

		// Swap successful! Exit.
		System.exit(0);
	}

	// Code adapted directly from net.minecraft.client.minecraft.java; should be
	// 100% accurate compared with MC.
	public static File getMinecraftDir() {

		// Get the operating system; this affects where .minecraft is
		String os = "Unknown";
		String osRealName = System.getProperty("os.name").toLowerCase();
		if (osRealName.contains("win")) {
			os = "Windows";
		} else if (osRealName.contains("mac")) {
			os = "MacOS";
		} else if (osRealName.contains("solaris") || osRealName.contains("linux") || osRealName.contains("unix")) {
			os = "Linux";
		} else if (osRealName.contains("sunos")) {
			os = "Solarus";
		}

		// Get the user's home directory
		String userHomeDir = System.getProperty("user.home", ".");
		// This file will contain the minecraft dir
		File appDir = null;
		// Handle differently for different OS types.
		if (os.equals("Linux")) {
			appDir = new File(userHomeDir, ".minecraft/");
		} else if (os.equals("Windows")) {
			String appDataDir = System.getenv("APPDATA");
			if (appDataDir != null) {
				appDir = new File(appDataDir, ".minecraft/");
			} else {
				appDir = new File(userHomeDir, ".minecraft/");
			}
		} else if (os.equals("MacOS")) {
			appDir = new File(userHomeDir, "Library/Application Support/minecraft");
		} else {
			// Unknown or Solarus
			appDir = new File(userHomeDir, "minecraft/");
		}

		// Return the minecraft dir if it has been found
		if (appDir != null && appDir.exists()) {
			return appDir;
		} else {
			// Inform user of failure.
			JOptionPane
					.showMessageDialog(
							null,
							"<html>MCDitty's jar file swapper could not find your .minecraft folder.<br><br>You can still install the mcditty update manually: just go to your .minecraft folder, and rename the updated jar 'minecraft.jar', replacing your current minecraft.jar.");
			System.exit(1);
			return null;
		}
	}

}
